@@ -1,6 +1,11 @@
+from __future__ import unicode_literals
+import base64
 import os
 import hashlib
 import itsdangerous
+import py
+from logging import getLogger
+log = getLogger(__name__)
 
 class Auth:
     LOGIN_EXPIRATION = 60*60*10  # 10 hours
@@ -29,7 +34,7 @@ class Auth:
                 return authuser
             return None
         else:
-            if not val.startswith(authuser + "-"):
+            if not val.startswith(authuser.encode() + b"-"):
                 log.debug("mismatch credential for user %r", authuser)
                 return None
             return authuser
@@ -38,6 +43,8 @@ class Auth:
         hash = self.db.user_validate(user, password)
         if hash:
             pseudopass = self.signer.sign(user + "-" + hash)
+            pseudopass = pseudopass.decode("ascii")
+            assert py.builtin._istext(pseudopass)
             return {"password":  pseudopass,
                     "expiration": self.LOGIN_EXPIRATION}
 
@@ -48,7 +55,7 @@ class Auth:
         if not self.db.user_exists(user):
             return ["nouser", user]
         try:
-            auth_user = self.get_auth_user(userpassword)
+            self.get_auth_user(userpassword)
         except self.Expired:
             return ["expired", user]
         else:
@@ -58,12 +65,12 @@ class Auth:
 
 def getpwhash(password, salt):
     hash = hashlib.sha256()
-    hash.update(salt)
-    hash.update(password)
+    hash.update(salt.encode("ascii"))
+    hash.update(password.encode("utf-8"))
     return hash.hexdigest()
 
 def newsalt():
-    return os.urandom(16).encode("base_64")
+    return py.builtin._totext(base64.b64encode(os.urandom(16)), "ascii")
 
 def verify_password(password, hash, salt):
     if getpwhash(password, salt) == hash:
