@@ -1,21 +1,19 @@
-
 """
 interact/control devpi-server background process.
 """
-import sys, os, time
+from __future__ import unicode_literals
+import time
 import py
-if sys.version_info >= (3, 0):
-    from urllib import parse as urlp
-else:
-    import urlparse as urlp
+
+from devpi_common.url import urlparse
 
 from devpi_server.vendor.xprocess import XProcess
-import requests
+from devpi_common.request import new_requests_session
 
 default_rooturl = "http://localhost:3141"
 
 def getnetloc(url, scheme=False):
-    parsed = urlp.urlparse(url)
+    parsed = urlparse(url)
     netloc = parsed.netloc
     if netloc.endswith(":80"):
         netloc = netloc[:-3]
@@ -40,10 +38,11 @@ class BackgroundServer:
     def _waitup(self, url, count=500):
         # try for 20 seconds to start devpi-server (which remotely
         # receives a serials list which may take a while)
+        req = new_requests_session(proxies=False)
         while count > 0:
             try:
-                r = requests.get(url)
-            except requests.exceptions.ConnectionError as e:
+                req.get(url)
+            except req.ConnectionError:
                 time.sleep(0.1)
                 count -= 1
             else:
