@@ -1,11 +1,12 @@
-import sys
+from __future__ import unicode_literals
+import base64
 import os.path
 import logging
 from logging import getLogger, basicConfig
 import argparse
 
 import py
-from devpi_server.types import canraise, cached_property
+from devpi_common.types import cached_property
 import devpi_server
 log = getLogger(__name__)
 
@@ -22,9 +23,7 @@ def addoptions(parser):
 
     web.addoption("--port",  type=int,
             default=3141,
-            help="port to listen for http requests.  When used with "
-                 "--gendeploy, port+1 will be used to prevent "
-                 "accidental clashes with ad-hoc runs.")
+            help="port to listen for http requests.")
 
     web.addoption("--outside-url",  type=str, dest="outside_url",
             metavar="URL",
@@ -60,7 +59,7 @@ def addoptions(parser):
             help="show devpi_version (%s)" % devpi_server.__version__)
 
     deploy.addoption("--gendeploy", action="store", metavar="DIR",
-            help="(unix only) install and generate a pre-configured "
+            help="(unix only, deprecated) install and generate a pre-configured "
                  "virtualenv directory which puts devpi-server "
                  "under supervisor control and provides some example "
                  "files for nginx/crontab.  If the specified DIR exists "
@@ -194,7 +193,7 @@ class Config:
     def secret(self):
         if not self.secretfile.check():
             self.secretfile.dirpath().ensure(dir=1)
-            self.secretfile.write(os.urandom(32).encode("base64"))
+            self.secretfile.write(base64.b64encode(os.urandom(32)))
             s = py.std.stat
             self.secretfile.chmod(s.S_IRUSR|s.S_IWUSR)
         return self.secretfile.read()
@@ -222,8 +221,10 @@ def render_string(confname, format=None, **kw):
     template = confname + ".template"
     from pkg_resources import resource_string
     templatestring = resource_string("devpi_server.cfg", template)
+    if not py.builtin._istext(templatestring):
+        templatestring = py.builtin._totext(templatestring, "utf-8")
 
-    kw = dict([(x[0],str(x[1])) for x in kw.items()])
+    kw = dict((x[0], str(x[1])) for x in kw.items())
     if format is None:
         result = templatestring.format(**kw)
     else:
