@@ -71,17 +71,16 @@ def apireturn(code, message=None, result=None, type=None):
     if message:
         d["message"] = message
     data = json.dumps(d, indent=2) + "\n"
-    header = meta_headers.copy()
-    header[str("content-type")] = "application/json"
-    raise HTTPResponse(body=data, status=code, header=header)
+    headers = meta_headers.copy()
+    headers[str("content-type")] = "application/json"
+    raise HTTPResponse(body=data, status=code, headers=headers)
 
 def json_preferred():
     # XXX do proper "best" matching
     return "application/json" in request.headers.get("Accept", "")
 
-def html_preferred():
-    accept = request.headers.get("Accept", "")
-    return not accept or "text/html" in accept
+def html_preferred(accept):
+    return not accept or "text/html" in accept or "*/*" in accept
 
 route = lazydecorator()
 
@@ -425,7 +424,7 @@ class PyPIView:
                     if not metadata:
                         abort_custom(400, "could not process form metadata")
                 res = stage.store_releasefile(name, version,
-                                              content.filename, content.value)
+                                              content.filename, content.file.read())
                 if res == 409:
                     abort(409, "%s already exists in non-volatile index" %(
                          content.filename,))
@@ -438,11 +437,12 @@ class PyPIView:
                             (jenkinurl,))
             else:
                 # docs have no version (XXX but they are tied to the latest)
-                if len(content.value) > MAXDOCZIPSIZE:
+                doczip = content.file.read()
+                if len(doczip) > MAXDOCZIPSIZE:
                     abort_custom(413, "zipfile size %d too large, max=%s"
-                                   % (len(content.value), MAXDOCZIPSIZE))
+                                   % (len(doczip), MAXDOCZIPSIZE))
                 stage.store_doczip(name, version,
-                                   py.io.BytesIO(content.value))
+                                   py.io.BytesIO(doczip))
         else:
             abort(400, "action %r not supported" % action)
         return ""
@@ -499,7 +499,7 @@ class PyPIView:
         name = ensure_unicode(name)
         info = stage.get_project_info(name)
         real_name = info.name if info else name
-        if html_preferred():
+        if html_preferred(request.headers.get("Accept")):
             # we need to redirect because the simple pages
             # may return status codes != 200, causing
             # pip to look at the full simple list at the parent url
@@ -821,8 +821,9 @@ def trigger_jenkins(stage, jenkinurl, testspec):
     source = render_string("devpibootstrap.py",
         INDEXURL=baseurl + stage.name,
         VIRTUALENVTARURL= (baseurl +
-            "root/pypi/+f/3a04aa2b32c76c83725ed4d9918e362e/"
-            "virtualenv-1.10.1.tar.gz"),
+            "root/pypi/+f/d3d/915836c1ada1be731ccaa12412b98/"
+            "virtualenv-1.11.2.tar.gz",
+            ),
         TESTSPEC=testspec,
         DEVPI_INSTALL_INDEX = baseurl + stage.name + "/+simple/"
     )
@@ -846,6 +847,7 @@ def trigger_jenkins(stage, jenkinurl, testspec):
     else:
         log.error("%s: failed to trigger jenkins at %s", r.status_code,
                   jenkinurl)
+        log.debug(r.content)
         return -1
 
 def abort_if_invalid_filename(name, filename):
